#
# list_cgi.py - List CGI methods.
# Copyright (c) 2005 shinGETsu Project.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: list_cgi.py,v 1.19 2005/10/28 13:01:41 fuktommy Exp $
#

import re
import cgi

import config
import gateway
from cache import *

__version__ = "$Revision: 1.19 $"


class CGI(gateway.CGI):

    """Class for /list.cgi."""

    appli_type = "list"

    def run(self):
        path = self.path_info()

        if path == "":
            path = config.menu

        form = cgi.FieldStorage(environ=self.environ, fp=self.stdin)
        if form.getfirst("cmd", "") == "new" and \
           form.getfirst("file", "").startswith("list_") and \
           self.environ["REQUEST_METHOD"] == "POST":
            flag = self.do_new_element(path, form)
            if not flag:
                return
            datfile = form.getfirst("file", "")
            title = self.str_encode(self.file_decode(datfile))
            self.print302(self.list_cgi + self.sep + title)
            return

        self.print_list(path)


    def print_list(self, path):
        cache = Cache(self.file_encode("list", path))
        if cache.has_record():
            pass
        elif self.check_get_cache():
            self.get_cache(cache)
        else:
            self.print404()
            return
        self.header(path)
        if self.isadmin:
            self.stdout.write(
                '<form method="get" action="' + self.admin_cgi + '">\n' +
                '<p>\n' +
                '  <input type="hidden" name="cmd" value="delete" />\n' +
                '  <input type="hidden" name="file" value="%s" />\n' %
                cache.datfile +
                '</p>\n')
        self.stdout.write('<ul>\n')
        buf = []
        stamp = {}
        for rec in cache.load_body():
            (bufstr, st) = self.record_to_string(cache, rec)
            buf.append(bufstr)
            stamp[bufstr] = st
        buf.sort(lambda a,b: cmp(stamp[b], stamp[a]))
        for recstr in buf:
            self.stdout.write(recstr+"\n")
        self.print_form(cache)
        self.menubar()
        self.remove_file_form(cache)
        self.footer()

    def record_to_string(self, cache, rec):
        stamp = rec["stamp"]

        buf = "  <li>\n"
        if self.isadmin:
            buf += '    <input type="radio" name="record"' + \
                   ' value="' + rec["stamp"] + "_" + rec["id"] + '"' + \
                   ' tabindex="1" accesskey="s" />\n'

        if ("type" in rec) and ("link" in rec):
            child_datfile = self.file_encode(rec["type"], rec["link"])
            child = Cache(child_datfile)
            if len(child) > 0:
                records = str(len(child))
            else:
                records = "?"

            if child.stamp > 0:
                stamp = child.stamp

            chstamp = self.localtime(stamp)
            chname = self.str_encode(rec["link"])
            if rec["type"] in config.types:
                anchor = '<a href="' + self.appli[rec["type"]] + self.sep + \
                         chname + '">' + rec["link"] + "</a>"
                msgtype = self.message[rec["type"]]
            else:
                anchor = rec["link"]
                msgtype = rec["type"]


            buf += '    ' + chstamp + "\n" + \
                   '    ' + anchor + "\n" + \
                   '    (' + msgtype + "/" + records + "/" + \
                   str(child.size*10/1024/1024/10.0) + \
                   self.message["mb"] + ")\n"

	if ("remove_stamp" in rec) and ("remove_id" in rec):
            chstamp = self.localtime(rec["remove_stamp"])
            buf += "    " + chstamp + " " + self.message["remove"] + ":\n"

        if "name" in rec:
            buf += "    " + rec["name"] + ":\n"

        if "pubkey" in rec:
            buf += '    <span class="sign" title="[%s]">%s</span>' % \
                   (rec["target"], rec["pubkey"])

        if "body" in rec:
            buf += "    " + rec["body"] + "\n"

        buf += "  </li>\n"
        return (buf, int(stamp))

    def print_form(self, cache):
        self.stdout.write("</ul>\n")
        if self.isadmin:
            self.stdout.write(
                "<p>\n" + 
                '  <input type="submit" value="%s"' %
                self.message["del_record"] +
                ' tabindex="2" accesskey="d" />\n' +
                "</p>\n</form>\n")

        if cache.size <= config.file_limit*1024*1024:
            self.print_new_element_form(parent=cache)
